using Polly;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.

builder.Services.AddSingleton<IAsyncPolicy<HttpResponseMessage>>(Policy.HandleResult<HttpResponseMessage>(r => !r.IsSuccessStatusCode)
.WaitAndRetryAsync(3, retryAttempt => TimeSpan.FromSeconds(retryAttempt)));

HttpClient httpClient = new HttpClient();
httpClient.BaseAddress = new System.Uri(@"http://localhost:5000/");

var app = builder.Build();

// Endpoint that fails 75% of the time
int _requestCount = 0; 

app.MapGet("/Inventory/{id}", (int id) =>
{
    _requestCount++;

    if (_requestCount % 4 == 0) // only one of out four requests will succeed
    {
        System.Console.WriteLine($"{_requestCount} It worked!");
        return Results.Ok(15);
    }

    System.Console.WriteLine($"{_requestCount} Something went wrong");
    return Results.Problem("Something went wrong");
});


// Endpoint that takes a Polly policy by DI
app.MapGet("/Catalog/{id}", async (int id, IAsyncPolicy<HttpResponseMessage> policy) =>
{
    string requestEndpoint = $"Inventory/{id}";

    HttpResponseMessage response = await policy.ExecuteAsync(() => httpClient.GetAsync(requestEndpoint));

    if (response.IsSuccessStatusCode)
    {
        int itemsInStock = await response.Content.ReadFromJsonAsync<int>();
        return Results.Ok(itemsInStock);
    }
    return Results.Problem("Something went wrong calling the inventory");

});

app.Run();